#!BPY

#blender api:
#https://docs.blender.org/api/

""" Registration info for Blender menus:
Name: 'Doug Dark Omen(.m3d)...'
Blender: 244
Group: 'Import'

Tip: 'Import from Dark Omen mesh format.'
"""

import Blender
from struct import pack, unpack
import os.path
import math

from m3d_common import *
from Blender import Mesh
from Blender.Mathutils import *
from Blender import Window

#defaultPath = r"C:\Program Files (x86)\Dark Omen\GameData\1pbat\*.m3d"
#defaultPath=r"C:\Users\sound\Documents\Blender documents\Heads\*.m3d"
defaultPath=r"C:\*.m3d"

#// Load the mesh
#lpUseMesh = appMeshes.Alloc();
#lpUseMesh->Load( szMeshname, szObjectname )

saved_pivot=M3DPOINT2()

def ToRadians(x):
        return (x*math.pi / 180)

#always loads the first mesh(e.g. szObjectname==NULL)
def Load_m3d(filename):
        texturePath1 = os.path.dirname(filename)+"/../textures/"
        texturePath = os.path.dirname(filename)+"/textures/"

	#short h 2bytes
	#long l 4bytes
	#char c 1byte
	#float f 4bytes
	#docs:
	#https://docs.python.org/3/library/struct.html

	#padding: https://en.wikipedia.org/wiki/Data_structure_alignment

	#load mesh

	num_textures = 0

	#read file header
	#read header
	if not filename.lower().endswith('.m3d'):
            print "Not an .m3d file"

    	F = file(filename,"rb")
    	
	#load file header
	m3dfh = M3DFILEHEADER()
	m3dh = M3DHEADER()

	values = unpack("<5l",F.read(20))
	m3dfh.ID=values[0]
	m3dfh.Magic=values[1]
	m3dfh.Version=values[2]
	m3dfh.CRC=values[3]
	m3dfh.NOT_CRC=values[4]
	
	#load header
	values=unpack("<2h",F.read(4))
	m3dh.numTextures=values[0]
	m3dh.numObjects=values[1]

	#count faces that use each texture,
	#a number of faces per texture
	TexFaceCounts = [] #lpbTexCounts #this actually holds index' of face lists
			   #rather than a number of faces
			   #so indirectly holds number of faces per texture using Face list, number
			   #is stored in the FaceList object instead.
	TextureList = [] #lpm3dt

	if m3dh.numTextures:
		for i in range(0,m3dh.numTextures):
			#read the next texture
			#read path(64 char)
			m3dt = M3DTEXTURE()
			m3dt.path = F.read(64)
        		m3dt.path = m3dt.path[:m3dt.path.find('\0')]

        		print "Texture path "+m3dt.path

			#read name(32 char)
			m3dt.name = F.read(32)
        		m3dt.name = m3dt.name[:m3dt.name.find('\0')]

        		print "Texture name "+m3dt.name

			TextureList.append(m3dt) #lpm3dt
			TexFaceCounts.append(0) #0 at this time #lpbTexCounts

		print "Num textures "+str(m3dh.numTextures)

	#Find Object
	bFound=False
	m3do = M3DOBJECTDESC()

	print "number of objects "+str(m3dh.numObjects)

	for i in range(0,m3dh.numObjects):
		
		m3do.FromStream(F)

		if m3do.name!="":
			bFound=True
			break
		#for each face, seek read face
		for n in range(0,m3do.faces):
                        tmp_face=M3DFACE()
                        tmp_face.FromStream(F)
		#F.seek(,1)
		#for each vertex, seek read vertex
                for n in range(0,m3do.vertices):
                        tmp_vertex=M3DVERTEX()
                        tmp_vertex.FromStream(F)

	if bFound==False:
		#no more objects
		return

	#Found an object
	#Some files contain null objects, do not continue if object is null
	if (m3do.vertices<=0) and (m3do.faces<=0):
                print "null object"
		return

	#record pivot point
	saved_pivot.x=m3do.pivot.x
	saved_pivot.y=m3do.pivot.y
	saved_pivot.z=m3do.pivot.z

	#record current position to use as seek position
	#to read faces again
	saveptr=F.tell()
		
	#//face list for each texture, comment doug
	#cTex is the number of face lists==number of textures
	#I think face lists are used for doing some u,v corrections
	
	#count number
	#number_of_textures=0

	number_of_face_lists=0#(cTex)

	if(len(TexFaceCounts)>0):
		for i in range(0,m3do.faces):
			f=M3DFACE()
			#read the face from file
			f.FromStream(F)

			print "Texture used by face "+str(f.texture)

			#if there are no faces associated with texture, then we've found a
			#new texture, so increase texture count
			if(TexFaceCounts[f.texture]==0):
				#number_of_textures+=1
				number_of_face_lists+=1
		
			#increase count of faces that use the texture because
			#this face uses it.
			#I can't remember why I need to do this(*keep count), 
			#oh aye this
			#allows to count number of face lists needed, basically
			#the code above will check if count==0
			#we need another face list,
			#by setting value++ we can tell if a face list has already been
			#counted

			TexFaceCounts[f.texture]+=1

	#if there are no textures, then use a default one,
	#set count to 1

	#for now assume there is always a texture

	#number of vertices is stored in the object desc
	iNumVertices=m3do.vertices
	#I've got number_of_face_lists(cTex)
	
	#face list for each texture(comment)

	#basically a FaceList per texture, think of a texture, usually
	#an m3d only has one, now think about a list of faces
	#those faces will display that texture
	FaceLists = [FaceList()]*number_of_face_lists

	#generate face lists
	facelist_index=0
	for i in range(0,m3dh.numTextures):
		#FaceLists[facelist_index]=FaceList()
		#yes a face list per texture??? unless texture is not used,

		if TexFaceCounts[i]>0: 
		    #TexFaceCounts holds number of faces that use each texture
		    #this texture has faces associated with it(face count>0)

			#this texture i has faces associated with it
			#setup the face list
			FaceLists[facelist_index]=FaceList()
			FaceLists[facelist_index].num_indices=0
			FaceLists[facelist_index].face_indices=[0]*(TexFaceCounts[i]*3)
			#assign the texture name to the face list, list
			#of faces that use the texture
			str_name=TextureList[i].name
			str_name=str_name.replace(".tga",".bmp")
			FaceLists[facelist_index].szTextureName=str_name
			
			#load the texture into blender now!
			print "Texture name:"+FaceLists[facelist_index].szTextureName
			#try path1
			try:
                                #texturePath="duff"
                                #texname = os.path.join("duff", "", str_name)
                                texname=os.path.join(texturePath1, "", str_name)
                                print "Loading img: "+texname
                                t = Blender.Image.Load(texname)
                        except:
                                texname = os.path.join(texturePath, "", str_name)
                                print "Loading img: "+texname
                                t = Blender.Image.Load(texname)

			
			FaceLists[facelist_index].lpTexture=t
			TextureList[i]=t
			
			#FaceLists[facelist_index].texture=t#todo

			#face list has been created for this texture
			#something else?
			#associate the face list(facelist_index) with the list""
			TexFaceCounts[i]=facelist_index
			facelist_index+=1
	#load faces to
	#face lists
	#back to start of faces
	F.seek(saveptr,0)

	print "Face lists "+str(len(FaceLists))

	#add a mesh to the scene
    	mesh = Blender.Mesh.New(m3do.name)

	for i in range(0,m3do.faces):

                print "faces="+str(m3do.faces)
                print "i="+str(i)
                
		f = M3DFACE()
		f.FromStream(F)
		
		if len(TexFaceCounts) > 0:
			list=TexFaceCounts[f.texture]#1 face list per texture
		else:
			list=0

                print "list "+str(list)
			
		idx=FaceLists[list].num_indices#could be 0 at this point because
					       #no faces have been added to the face list
		#i think the purpose of face lists is to associate
		#faces with textures, faces exist on their own but
		#a face list has the texture used by each face

                print "number of indices "+str(len(FaceLists[list].face_indices))
                print "start idx "+str(idx)
		
		FaceLists[list].face_indices[idx+0]=f.vertex[0]
		FaceLists[list].face_indices[idx+1]=f.vertex[1]
		FaceLists[list].face_indices[idx+2]=f.vertex[2]
		FaceLists[list].num_indices+=3
		

	#load vertices
	vertices = [M3DVERTEX()]*(m3do.vertices+1)

        #F.seek(-1,1)#-1 relative to current
	for i in range(1,m3do.vertices+1):
		m3dv=M3DVERTEX()
		m3dv.FromStream(F)
		vertices[i]=m3dv#I don't need D3DLVERTEX, that's only for d3d applications
		if i==0:
                        print "u0 "+str(m3dv.u)+"v0 "+str(m3dv.v)

	#Position vertices using pivot
	#If mesh needs to be relocated, do so now
	if m3do.flags & M3D_PIVOT_RELATIVE:
		for i in range(0,m3do.vertices+1):
			vertices[i].vertex.x += saved_pivot.x
			vertices[i].vertex.y += saved_pivot.y
			vertices[i].vertex.z += saved_pivot.z

        #add 1
	#mesh.verts.extend([(0,0,vertices[i].vertex.z)])
	mesh.verts.extend([(0,0,0)])
	
	#add vertices
    	for i in range(1,m3do.vertices+1):
                #vert=[0]*3
                #vert[0]=vertices[i].vertex.x
                #vert[1]=vertices[i].vertex.y
                #vert[2]=vertices[i].vertex.z
                mesh.verts.extend([(-vertices[i].vertex.x,vertices[i].vertex.y,vertices[i].vertex.z)])

	#vertices are loaded
	F.seek(saveptr,0)
	for i in range(0,m3do.faces):

                print "faces="+str(m3do.faces)
                print "i="+str(i)
                
		f = M3DFACE()
		f.FromStream(F)

                print "size "+str(len(f.vertex))
		
                #start
                #mesh.faces.extend(
                            #f.vertex[0],
                            #mesh.verts[face.indexTuple.t3],
                            #mesh.verts[face.indexTuple.t2]
                            #f.vertex[1],
                            #f.vertex[2]
                            #)
                #mesh.faces.extend[(f.vertex[2],f.vertex[1],f.vertex[0])]
                mesh.faces.extend(
                            f.vertex[2]+1,
                            f.vertex[1]+1,
                            f.vertex[0]+1
                            )
                
                mesh.faces[i].image=TextureList[f.texture]
                #mesh.faces[i].no.x=-mesh.faces[i].no.x
                #mesh.faces[i].no.y=-mesh.faces[i].no.y
                #mesh.faces[i].no.z=-mesh.faces[i].no.z
                        
                uv=[0.0]*2
                uv[0]=vertices[f.vertex[0]+1].u
                uv[1]=vertices[f.vertex[0]+1].v
                uv1=Blender.Mathutils.Vector( uv[0], 1-uv[1] )
                        
                uv[0]=vertices[f.vertex[1]+1].u
                uv[1]=vertices[f.vertex[1]+1].v
                uv2=Blender.Mathutils.Vector( uv[0], 1-uv[1] )
                        
                uv[0]=vertices[f.vertex[2]+1].u
                uv[1]=vertices[f.vertex[2]+1].v
                #uv3=uv.AsBlenderVector()
                uv3=Blender.Mathutils.Vector( uv[0], 1-uv[1] )
                        
                mesh.faces[i].uv = (uv3,uv2,uv1)

                nor=Blender.Mathutils.Vector( f.normal.x, f.normal.y, f.normal.z )
                #mesh.faces[i].no.x=nor.x
                #mesh.faces[i].no.y=nor.y
                #mesh.faces[i].no.z=nor.z
                #end

	#ah ah!
	#idx is zero at start for face list then after adding to the list
	#that's how idx+0,idx+1,idx+2 works
	#idx becomes +3 every time a face is added to that list

	#what is FaceLists used for?
	#maybe I can just use current code for blender
	#try set vertex color in blender import, maybe color is stored in m3d
	# could be why faces appear lit up

	#FaceLists[0] is the first face list, notice 0, that's why
	#for each face, save it's indices to blender

	#for each vertex, save vertex to blender
	#save position
	#save u,v
	#save colour
	

    	F.close()

        #add faces
        #for i in range(0,len(FaceLists)):
                #for j in range(0, FaceLists[i].num_indices/3):
                        #add face
                        
                        #mesh.faces.extend(
                            #FaceLists[i].face_indices[j*3+2],
                            #mesh.verts[face.indexTuple.t3],
                            #mesh.verts[face.indexTuple.t2]
                            #FaceLists[i].face_indices[j*3+1],
                            #FaceLists[i].face_indices[j*3+0]
                            #)
                        #mesh.faces[j].image=FaceLists[i].lpTexture
                        
                        #uv=[0]*2
                        #uv[0]=vertices[FaceLists[i].face_indices[j*3+2]].u
                        #uv[1]=vertices[FaceLists[i].face_indices[j*3+2]].v
                        #uv1=Blender.Mathutils.Vector( uv[0], 1-uv[1] )
                        
                        #uv[0]=vertices[FaceLists[i].face_indices[j*3+1]].u
                        #uv[1]=vertices[FaceLists[i].face_indices[j*3+1]].v
                        #uv2=Blender.Mathutils.Vector( uv[0], 1-uv[1] )
                        
                        #uv[0]=vertices[FaceLists[i].face_indices[j*3+0]].u
                        #uv[1]=vertices[FaceLists[i].face_indices[j*3+0]].v
                        #uv3=uv.AsBlenderVector()
                        #uv3=Blender.Mathutils.Vector( uv[0], 1-uv[1] )
                        
                        #mesh.faces[j].uv = (uv1,uv2,uv3)
        #mesh.verts.delete(0)                

        #add mesh to scene
        #use texture
        mesh.faceUV = True
        #add mesh to scene
	scene = Blender.Scene.GetCurrent()
       	blenderObject = scene.objects.new(mesh, m3do.name)
       	angles=[0]*3
       	angles[0]=ToRadians(90)#x
       	angles[1]=ToRadians(0)#y
       	angles[2]=ToRadians(180)#z
       	blenderObject.setEuler(angles)
        #mesh.remDoubles(0.05)

       	#add cube at pivot point
       	me = Mesh.Primitives.Cube(2.0)   # create a new cube of size 2
       	v=Blender.Mathutils.Vector( saved_pivot.x, saved_pivot.z, saved_pivot.y )
       	t=TranslationMatrix(v)
       	me.transform(t)
        sc = Blender.Scene.GetCurrent()          # get current scene
        sc.objects.new(me,'Mesh')        # add a new mesh-type object to the scene
        Window.RedrawAll()

arg = __script__['arg']

if __name__ == '__main__':
    Blender.Window.FileSelector(Load_m3d, "Import Dark Omen mesh", defaultPath)
