import Blender
from struct import pack, unpack

M3D_PIVOT_RELATIVE=2

class M3DTEXTURE(object):
	
	def __init__(self):
        	self.path = "" #64 chars
        	self.name = "" #32 chars

class M3DFILEHEADER(object):
	
	def __init__(self):
        	self.ID = 0
        	self.Magic = 0
        	self.Version = 0
        	self.CRC = 0
        	self.NOT_CRC = 0

class M3DHEADER(object):
	
	def __init__(self):
        	self.numTextures = 0
        	self.numObjects = 0

class M3DPOINT(object):
	def __init__(self):
        	self.x = 0
        	self.y = 0
        	self.z = 0

class M3DVERTEX(object):
	def __init__(self):
        	self.vertex = M3DPOINT()#float
        	self.normal = M3DPOINT()#float
        	self.rgba = 0#long
		self.u=0#float
		self.v=0#float
		self.spare1=0#long
		self.spare2=0#long

	def FromStream(self, F):

                #size before compilation: 12+12+4+4+4+4+4
                #sizeof()=44
                #good no padding needed

		#read three floats
		values=unpack("<3f",F.read(12))
		self.vertex.x=values[0]
		self.vertex.y=values[1]
		self.vertex.z=values[2]

		#read three floats
		values=unpack("<3f",F.read(12))
		self.normal.x=values[0]
		self.normal.y=values[1]
		self.normal.z=values[2]

		self.rgba=unpack("<1l",F.read(4))[0]
		#padding=unpack("<1h",F.read(2))[0]

		#read 2 floats
		self.u=unpack("<1f",F.read(4))[0]
		self.v=unpack("<1f",F.read(4))[0]

		self.spare1=unpack("<1l",F.read(4))[0]
		self.spare2=unpack("<1l",F.read(4))[0]
        	
        	return

class M3DFACE(object):
	def __init__(self):
        	self.vertex = [] #short
        	self.texture = 0 #short
        	self.normal = M3DPOINT()
		self.spare1 = 0 #long
		self.spare2 = 0 #long

	def FromStream(self, F):

                #size before compilation:6+2+12+4+4
                #sizeof=28
                #good, no padding

		#read three shorts
		values=unpack("<3h",F.read(6))
		self.vertex.append(values[0])
		self.vertex.append(values[1])
		self.vertex.append(values[2])

		print "indices "+str(values[0])+" "+str(values[1])+" "+str(values[2])+" "

		self.texture=unpack("<1h",F.read(2))[0]

		#read three floats
		values=unpack("<3f",F.read(12))
		self.normal.x=values[0]
		self.normal.y=values[1]
		self.normal.z=values[2]

		#read 2 longs
		self.spare1=unpack("<1l",F.read(4))[0]
		self.spare2=unpack("<1l",F.read(4))[0]
        	
        	return

class M3DPOINT2(object):
	def __init__(self):
        	self.x = 0
        	self.y = 0
        	self.z = 0
		self.flags=0

class FaceList(object):
	def __init__(self):
        	self.face_indices = []#each entry is in 3s of course, use a face list
					#to set faces and their textures in blender
		self.num_indices=0
		self.szTextureName=""
		self.lpTexture=0

class M3DOBJECTDESC(object):
	
	def __init__(self):
        	self.name = ""
        	self.parent = 0
        	self.pivot = M3DPOINT()
        	self.vertices = 0#number of vertices
        	self.faces = 0
		self.flags = 0
		self.spare1 = 0
		self.spare2 = 0

	def FromStream(self, F):
                #the size of any structure should be padded to meet a size
                #that is a multiple of the highest alignment, e.g. 4
                #if one float is used and 2 shorts in the same structure

                #size of M3DOBJECTDESC before compilation: 32+2+12+2+2+4+4+4
                #size=62
                #size after compilation: 64, so must be 2 bytes padding
                
		#read name(32 char)
		#name = F.read(16)
                name=''.join(unpack("<32c",F.read(32)))
        	self.name = name[:name.find('\0')]

        	print "object name "+self.name

		self.parent=unpack("<1h",F.read(2))[0]
		padding=unpack("<1h",F.read(2))[0]

		print "parent "+str(self.parent)

		#values=unpack("<3f",F.read(12))
		self.pivot.x=unpack("<1f",F.read(4))[0]#I want 9.55 for x
		self.pivot.y=unpack("<1f",F.read(4))[0]
		self.pivot.z=unpack("<1f",F.read(4))[0]

		print "pivot "+str(self.pivot.x)+" "+str(self.pivot.y)+" "+str(self.pivot.z)

		self.vertices=unpack("<1h",F.read(2))[0]
		self.faces=unpack("<1h",F.read(2))[0]
		self.flags=unpack("<1l",F.read(4))[0]
		self.spare1=unpack("<1l",F.read(4))[0]
		self.spare2=unpack("<1l",F.read(4))[0]
        	
        	return
